@@ -25,17 +25,7 @@
 # exception to your version of the code, but you are not obligated to 
 # do so. If you do not wish to do so, delete this exception statement 
 # from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
 
-outpath=$(echo "$1" | sed "s/.po/\/LC_MESSAGES/" -)
+outpath=$(echo "$1" | sed "s/.po/\/LC_MESSAGES/")
 mkdir -p $outpath
 msgmerge -o - $1 messages.pot | msgfmt -c -o $outpath/exaile.mo -
