@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,19 +15,23 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-from xl.nls import gettext as _
 
-import logging, threading, time
+import logging
+import threading
+import time
+
 import gst, gobject
+
+from xl.nls import gettext as _
 from xl import event, settings, common
 from xl.player import _base, pipe
 
@@ -45,7 +49,7 @@ class UnifiedPlayer(_base.ExailePlayer):
         #   44100hz audio all the time.
         #   Or better yet, fix gst to handle changing caps :D
         self.caps = gst.Caps(
-                "audio/x-raw-int, " 
+                "audio/x-raw-int, "
                 "endianness=(int)1234, "
                 "signed=(boolean)true, "
                 "width=(int)16, "
@@ -55,7 +59,7 @@ class UnifiedPlayer(_base.ExailePlayer):
         self.pipe = gst.Pipeline()
         self.adder = gst.element_factory_make("adder")
         self.audio_queue = gst.element_factory_make("queue")
-    
+
         self.streams = [None, None]
 
         self._load_queue_values()
@@ -70,13 +74,13 @@ class UnifiedPlayer(_base.ExailePlayer):
         # a decent buffer. This is done as a setting so users whose
         # collections are on slower media can increase it to preserve
         # gapless, at the expense of UI lag.
-        self.audio_queue.set_property("max-size-time", 
+        self.audio_queue.set_property("max-size-time",
                 settings.get_option("player/queue_duration", 100000))
 
     def _setup_pipeline(self):
         self.pipe.add(
-                self.adder, 
-                self.audio_queue, 
+                self.adder,
+                self.audio_queue,
                 self.mainbin
                 )
         self.adder.link(self.audio_queue)
@@ -95,7 +99,7 @@ class UnifiedPlayer(_base.ExailePlayer):
                 self.play(tr, user=False)
         else:
             self.unlink_stream(stream)
-    
+
     def setup_bus(self):
         """
             setup the gstreamer message bus and callacks
@@ -110,10 +114,12 @@ class UnifiedPlayer(_base.ExailePlayer):
             logger.warning("EOS: ", message)
         elif message.type == gst.MESSAGE_TAG and self.tag_func:
             self.tag_func(message.parse_tag())
-            if not self.current['__length']:
+            if not self.current.get_tag_raw('__length'):
                 try:
-                    duration = float(self.mainbin.query_duration(gst.FORMAT_TIME, None)[0])/1000000000
-                    if duration > 0: self.current['__length'] = duration
+                    duration = float(self.mainbin.query_duration(
+                        gst.FORMAT_TIME, None)[0])/1000000000
+                    if duration > 0:
+                        self.current.set_tag_raw('__length', duration)
                 except gst.QueryError:
                     logger.debug("Couldn't query duration via GStreamer")
         elif message.type == gst.MESSAGE_ERROR:
@@ -138,7 +144,7 @@ class UnifiedPlayer(_base.ExailePlayer):
             return self.streams[self._current_stream].get_position()
         except AttributeError:
             return 0
-        
+
     @common.synchronized
     def play(self, track, user=True):
         if not track:
@@ -170,7 +176,7 @@ class UnifiedPlayer(_base.ExailePlayer):
                 self.unlink_stream(self.streams[self._current_stream])
 
         self.streams[next] = AudioStream("Stream%s"%(next), caps=self.caps)
-        self.streams[next].dec.connect("drained", self._on_drained, 
+        self.streams[next].dec.connect("drained", self._on_drained,
                 self.streams[next])
 
         if not self.link_stream(self.streams[next], track):
@@ -187,16 +193,16 @@ class UnifiedPlayer(_base.ExailePlayer):
         if fading:
             timeout = int(float(duration)/float(100))
             if self.streams[next]:
-                gobject.timeout_add(timeout, self._fade_stream, 
+                gobject.timeout_add(timeout, self._fade_stream,
                         self.streams[next], 1)
             if self.streams[self._current_stream]:
-                gobject.timeout_add(timeout, self._fade_stream, 
+                gobject.timeout_add(timeout, self._fade_stream,
                         self.streams[self._current_stream], -1, True)
             if settings.get_option("player/crossfading", False):
-                time = int(track.get_duration()*1000 - duration)
-                gobject.timer_id = gobject.timeout_add(time, 
+                time = int(track.get_tag_raw("__length")*1000 - duration)
+                gobject.timer_id = gobject.timeout_add(time,
                         self._start_crossfade)
-                
+
         self._current_stream = next
         if not playing:
             event.log_event('playback_player_start', self, track)
@@ -210,7 +216,7 @@ class UnifiedPlayer(_base.ExailePlayer):
             return False
         else:
             return True
-        
+
     def _fade_stream(self, stream, direction, delete=False):
         current = stream.get_volume()
         current += direction/100.0
@@ -227,8 +233,8 @@ class UnifiedPlayer(_base.ExailePlayer):
         if self._timer_id:
             gobject.source_remove(self._timer_id)
         if tr is None:
-            self._timer_id = gobject.timeout_add(
-                    1000*(self.current.get_duration() - self.get_time()),
+            self._timer_id = gobject.timeout_add(1000 * \
+                    (self.current.get_tag_raw('__length') - self.get_time()),
                     self.stop)
         return False
 
@@ -240,7 +246,7 @@ class UnifiedPlayer(_base.ExailePlayer):
         if not settings.get_option("player/crossfading", False):
             return
         duration = settings.get_option("player/crossfade_duration", 3000)
-        time = int( self.current.get_duration()*1000 - \
+        time = int( self.current.get_tag_raw('__length')*1000 - \
                 (self.get_time()*1000 + duration) )
         if time < duration: # start crossfade now, we're late!
             gobject.idle_add(self._start_crossfade)
@@ -288,7 +294,7 @@ class UnifiedPlayer(_base.ExailePlayer):
         if self.is_playing() or self.is_paused():
             current = self.current
             self.pipe.set_state(gst.STATE_NULL)
-            for stream in self.streams:           
+            for stream in self.streams:
                 self.unlink_stream(stream)
             self._reset_crossfade_timer()
             event.log_event('playback_player_end', self, current)
@@ -306,7 +312,7 @@ class UnifiedPlayer(_base.ExailePlayer):
             event.log_event('playback_player_pause', self, self.current)
             return True
         return False
- 
+
     @common.synchronized
     def unpause(self):
         """
@@ -356,10 +362,10 @@ class AudioStream(gst.Bin):
         self.capsfilter = gst.element_factory_make("capsfilter")
         self.capsfilter.set_property("caps", self.caps)
         self.vol = gst.element_factory_make("volume")
-        self.add(self.dec, 
-                self.audioconv, 
-                self.audioresam, 
-                self.provided, 
+        self.add(self.dec,
+                self.audioconv,
+                self.audioresam,
+                self.provided,
                 self.capsfilter,
                 self.vol)
         self.audioconv.link(self.audioresam)
@@ -392,16 +398,16 @@ class AudioStream(gst.Bin):
         if track.is_local():
             if not track.exists():
                 logger.error("File does not exist: %s" %
-                        track.get_loc())
+                        track.get_loc_for_display())
                 return False
-        
+
         self.track = track
 
         uri = track.get_loc_for_io()
 
         logger.info("Playing %s" % uri)
         self.reset_playtime_stamp()
-        
+
         self.dec.set_property("uri", uri)
 
         # TODO: abstract this into generic uri handling via providers
@@ -414,7 +420,7 @@ class AudioStream(gst.Bin):
     def __notify_source(self, *args):
         # this is for handling multiple CD devices properly
         source = self.dec.get_property('source')
-        device = self.track.get_loc().split("#")[-1]
+        device = self.track.get_loc_for_io().split("#")[-1]
         source.set_property('device', device)
         self.dec.disconnect(self.notify_id)
 
@@ -423,7 +429,7 @@ class AudioStream(gst.Bin):
             updates the total playtime for the currently playing track
         """
         if self.track and self._playtime_stamp:
-            last = self.track['__playtime']
+            last = self.track.get_tag_raw('__playtime')
             if type(last) == str:
                 try:
                     last = int(last)
@@ -431,8 +437,8 @@ class AudioStream(gst.Bin):
                     last = 0
             elif type(last) != int:
                 last = 0
-            self.track['__playtime'] = last + int(time.time() - \
-                    self._playtime_stamp)
+            self.track.set_tag_raw('__playtime',
+                    last + int(time.time() - self._playtime_stamp) )
             self._playtime_stamp = None
 
     def reset_playtime_stamp(self):
@@ -478,7 +484,7 @@ class AudioStream(gst.Bin):
             return None
 
     def get_position(self):
-        if self.is_paused(): 
+        if self.is_paused():
             return self.last_position
         try:
             self.last_position = self.dec.query_position(gst.FORMAT_TIME)[0]
@@ -495,7 +501,7 @@ class AudioStream(gst.Bin):
             logger.debug("Failed to settle state on %s."%self)
             gst.Bin.set_state(self, gst.STATE_NULL)
             event.log_event("stream_settled", self, None)
-            return 
+            return
         gobject.idle_add(self._settle_state_sub)
 
     @common.threaded
@@ -524,7 +530,7 @@ class AudioStream(gst.Bin):
 
         value = int(gst.SECOND * value)
         seekevent = gst.event_new_seek(1.0, gst.FORMAT_TIME,
-            gst.SEEK_FLAG_FLUSH,gst.SEEK_TYPE_SET, value, 
+            gst.SEEK_FLAG_FLUSH,gst.SEEK_TYPE_SET, value,
             gst.SEEK_TYPE_NONE, 0)
 
         self.vol.send_event(seekevent)
@@ -536,7 +542,7 @@ class AudioStream(gst.Bin):
             internal code used if seek is called before the stream is ready
         """
         if self._settle_flag == 1 or object != self:
-            return 
+            return
         event.remove_callback(self._seek_delayed, type, object)
-        self._seek_event.set()    
+        self._seek_event.set()
 
