@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,33 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import gtk, pango, gtk.gdk, gobject
+import copy
+import logging
+import os
+import os.path
+import math
+import urllib
+
+import gobject
+import gtk
+import gtk.gdk
+import pango
+
 from xlgui import guiutil, menu, plcolumns
 from xlgui import rating
 from xlgui.plcolumns import *
-from xl import playlist, event, track, collection, xdg
-from xl import settings, trackdb
+from xl import playlist, event, collection, xdg, settings, trax
 from xl.nls import gettext as _
-import copy, urllib
-import logging
-import os, os.path
-import math
+
 logger = logging.getLogger(__name__)
 
 class Playlist(gtk.VBox):
@@ -53,7 +60,7 @@ class Playlist(gtk.VBox):
     default_columns = ['tracknumber', 'title', 'album', 'artist', '__length']
     menu_items = {}
     _is_drag_source = False
-    
+
     __gsignals__ = {
         'playlist-content-changed': (gobject.SIGNAL_RUN_LAST, None, (bool,)),
         'customness-changed': (gobject.SIGNAL_RUN_LAST, None, (bool,)),
@@ -75,10 +82,13 @@ class Playlist(gtk.VBox):
         self.player = self.exaile.player
         self.queue = queue
         self.search_keyword = ''
-        self.xml = main.xml
+        self.builder = main.builder
         self._initial_column_ids = _column_ids
         self._is_queue = _is_queue
 
+        self._redraw_queue = []
+        self._redraw_id = 0
+
         if not _is_queue:
             self.playlist = copy.copy(pl)
             self.playlist.ordered_tracks = pl.ordered_tracks[:]
@@ -87,7 +97,7 @@ class Playlist(gtk.VBox):
 
         # see plcolumns.py for more information on the columns menu
         if not Playlist.menu_items:
-            plcolumns.setup_menu(self.xml.get_widget('columns_menu_menu'),
+            plcolumns.setup_menu(self.builder.get_object('columns_menu_menu'),
                 Playlist.menu_items)
 
         self._setup_tree()
@@ -121,57 +131,74 @@ class Playlist(gtk.VBox):
         from xlgui import properties
         tracks = self.get_selected_tracks()
 
-        if len(tracks) == 0:
+        if not tracks:
             return False
 
-        dialog = properties.TrackPropertiesDialog(self.main.window, 
-            tracks[0])
-        result = dialog.run()
-        dialog.hide()
+        dialog = properties.TrackPropertiesDialog(self.main.window,
+            tracks)
+        #result = dialog.run()
+        #dialog.hide()
+
+        #return True
 
-        return True
-    
     def refresh_changed_tracks(self, type, track, tag):
-        tracks = [track]
-        if not tracks or tracks == [] or not \
-            settings.get_option('gui/sync_on_tag_change', True):
+        """
+            Called when a track is known to have a tag changed
+        """
+        if not track or not \
+            settings.get_option('gui/sync_on_tag_change', True) or not\
+            tag in self.get_column_ids():
             return
+        
+        if self._redraw_id:
+            gobject.source_remove(self._redraw_id)
+        self._redraw_queue.append(track)
+        self._redraw_id = gobject.timeout_add(100,
+                self.__refresh_changed_tracks)
+
+
+    def __refresh_changed_tracks(self):
+        tracks = {}
+        for tr in self._redraw_queue:
+            tracks[tr.get_loc_for_io()] = tr
+        self._redraw_queue = []
+
+        selection = self.list.get_selection()
+        info = selection.get_selected_rows()
+
         it = self.model.get_iter_first()
         while it:
-            cur = self.model.get_value(it, 0)
-            for track in tracks:
-                if cur.get_loc() == track.get_loc():
-                    self.update_iter(it, track)
-                    tracks.remove(track)
+            loc = self.model.get_value(it, 0).get_loc_for_io()
+            if loc in tracks:
+                self.update_iter(it, tracks[loc])
             it = self.model.iter_next(it)
         self.list.queue_draw()
 
+        if info:
+            for path in info[1]:
+                selection.select_path(path)
+
+    def selection_changed(self):
+        trs = self.get_selected_tracks()
+        self.builder.get_object('track_properties_item').set_sensitive(bool(trs))
+
     def on_stop_track(self, event, queue, stop_track):
         """
             Makes sure to select the next track in the
             playlist after playback has stopped due to SPAT
         """
         next_track = self.playlist.next()
-        iter = self.model.get_iter_first()
-        selection = self.list.get_selection()
-        selection.unselect_all()
-
-        while iter:
-            track = self.model.get_value(iter, 0)
-            if track == next_track:
-                path = self.model.get_path(iter)
-                selection.select_path(path)
-                break
-            iter = self.model.iter_next(iter)
+        next_index = self.playlist.index(next_track)
+        self.list.set_cursor(next_index)
 
     def queue_selected_tracks(self):
         """
             Toggles queue of selected tracks
         """
-        tracks = self.get_selected_tracks()
+        trs = self.get_selected_tracks()
 
         queue_tracks = self.queue.ordered_tracks
-        for track in tracks:
+        for track in trs:
             if track in queue_tracks:
                 queue_tracks.remove(track)
             else:
@@ -181,20 +208,36 @@ class Playlist(gtk.VBox):
         self.list.queue_draw()
 
     def set_rating(self, widget, rating):
-        tracks = self.get_selected_tracks()
+        trs = self.get_selected_tracks()
         steps = settings.get_option('miscellaneous/rating_steps', 5)
         r = float((100.0*rating)/steps)
-        for track in tracks:
-            track['__rating'] = r
+        for track in trs:
+            track.set_rating(rating)
         event.log_event('rating_changed', self, r)
 
+    def get_column_ids(self):
+        column_ids = None
+        if settings.get_option('gui/trackslist_defaults_set', False):
+            ids = settings.get_option("gui/columns", [])
+            # Don't add invalid columns.
+            all_ids = frozenset(self.COLUMNS.iterkeys())
+            column_ids = frozenset(id for id in ids if id in all_ids)
+
+        if not column_ids:
+            # Use default.
+            ids = self.default_columns
+            settings.set_option('gui/trackslist_defaults_set', True)
+            settings.set_option('gui/columns', ids)
+            column_ids = frozenset(ids)
+        return column_ids
+
     def _setup_col_menus(self):
         """
             Sets up the column menus (IE, View->Column->Track, etc)
         """
-        self.resizable_cols = self.xml.get_widget('col_resizable_item')
+        self.resizable_cols = self.builder.get_object('col_resizable_item')
         self.not_resizable_cols = \
-            self.xml.get_widget('col_not_resizable_item')
+            self.builder.get_object('col_not_resizable_item')
         if not self.resizable_cols and not self.not_resizable_cols:
             return # potentially dangerous if someone breaks the gladefile...
         self.resizable_cols.set_active(
@@ -202,18 +245,16 @@ class Playlist(gtk.VBox):
         self.not_resizable_cols.set_active(not \
             settings.get_option('gui/resizable_cols', False))
         self.resizable_cols.connect('activate', self.activate_cols_resizable)
-        self.not_resizable_cols.connect('activate',
-            self.activate_cols_resizable)
+        # activate_cols_resizable will be called by resizable_cols anyway, no need to call it twice
+        #self.not_resizable_cols.connect('activate',
+        #    self.activate_cols_resizable)
 
-        column_ids = None
+        column_ids = self.get_column_ids()
         if settings.get_option('gui/trackslist_defaults_set', False):
-            column_ids = set()
             ids = settings.get_option("gui/columns", [])
             # Don't add invalid columns.
-            all_ids = frozenset(self.COLUMNS.keys())
-            for id in ids:
-                if id in all_ids:
-                    column_ids.add(id)
+            all_ids = frozenset(self.COLUMNS.iterkeys())
+            column_ids = frozenset(id for id in ids if id in all_ids)
 
         if not column_ids:
             # Use default.
@@ -223,7 +264,7 @@ class Playlist(gtk.VBox):
             column_ids = frozenset(ids)
 
         if not self._is_queue:
-            for col_struct in self.COLUMNS.values():
+            for col_struct in self.COLUMNS.itervalues():
                 try:
                     menu = Playlist.menu_items[col_struct.id]
                 except KeyError:
@@ -238,8 +279,8 @@ class Playlist(gtk.VBox):
         """
             Filter the playlist with a keyword
         """
-        tracks = self.playlist.filter(keyword)
-        self._set_tracks(tracks)
+        trs = self.playlist.filter(keyword)
+        self._set_tracks(trs)
         self.search_keyword = keyword
 
     def change_column_settings(self, item, data):
@@ -269,12 +310,7 @@ class Playlist(gtk.VBox):
             Called when the user chooses whether or not columns can be
             resizable
         """
-        if 'not' in widget.name:
-            resizable = False
-        else:
-            resizable = True
-
-        settings.set_option('gui/resizable_cols', resizable)
+        settings.set_option('gui/resizable_cols', widget.get_active())
         self.emit('column-settings-changed')
 
     def update_col_settings(self):
@@ -283,6 +319,11 @@ class Playlist(gtk.VBox):
         """
         selection = self.list.get_selection()
         info = selection.get_selected_rows()
+        # grab the first visible raw of the treeview
+        firstpath = self.list.get_path_at_pos(4,4)
+        topindex = None
+        if firstpath:
+            topindex = firstpath[0][0]
 
         self.list.disconnect(self.changed_id)
         columns = self.list.get_columns()
@@ -293,42 +334,38 @@ class Playlist(gtk.VBox):
         self._set_tracks(self.playlist.get_tracks())
         self.list.queue_draw()
 
+        if firstpath:
+            self.list.scroll_to_cell(topindex)
         if info:
-            paths = info[1]
-            if paths:
-                if paths[0]:
-                    iter = self.model.get_iter(paths[0])
-                    track = self.model.get_value(iter, 0)
-                    index = self.playlist.index(track)
-                    self.list.scroll_to_cell(index)
-                    self.list.set_cursor(index)
-                
-                for path in paths:
-                    selection.select_path(path)
+            for path in info[1]:
+                selection.select_path(path)
 
     def on_remove_tracks(self, type, playlist, info):
         """
             Called when someone removes tracks from the contained playlist
         """
-        removed = info[2]
-        self.remove_tracks(removed)
+        start = info[0]
+        end = info[1]
+        paths = [(x,) for x in range(start, end-1)]
+        self.remove_rows(paths, playlist=False)
 
-    def on_add_tracks(self, type, playlist, tracks, scroll=False):
+    def on_add_tracks(self, type, playlist, trs, scroll=False):
         """
             Called when someone adds tracks to the contained playlist
         """
-        for track in tracks:
+        for track in trs:
             self._append_track(track)
-        
-        self.emit('track-count-changed', len(self.playlist))
+
+        newlength = len(self.playlist)
+        self.emit('track-count-changed', newlength)
         range = self.list.get_visible_range()
         offset = 0
         if range:
             offset = range[1][0] - range[0][0]
-        
-        if tracks and scroll:
+
+        if trs and scroll:
             try:
-                if offset > len(self.playlist):
+                if offset > newlength:
                     self.list.scroll_to_cell(self.playlist.index(tracks[-1]))
                 else:
                     self.list.scroll_to_cell(self.playlist.index(tracks[offset]))
@@ -336,14 +373,13 @@ class Playlist(gtk.VBox):
                 self.list.scroll_to_cell(self.playlist.index(tracks[0]))
         self.set_needs_save(True)
 
-    def _set_tracks(self, tracks):
+    def _set_tracks(self, trs):
         """
             Sets the tracks that this playlist should display
         """
-        self.list.set_model(self.model_blank)
         self.model.clear()
 
-        for track in tracks:
+        for track in trs:
             self._append_track(track)
 
         self.list.set_model(self.model)
@@ -357,7 +393,7 @@ class Playlist(gtk.VBox):
         count = 3
         search_column = settings.get_option("gui/search_column", "Title")
         for col in self.list.get_columns():
-            if col.get_title() == search_column:
+            if col.get_title().decode('utf-8') == search_column:
                 self.list.set_search_column(count)
             count = count + 1
 
@@ -367,14 +403,9 @@ class Playlist(gtk.VBox):
         """
         ar = [song, None, None]
         for field in self.append_map:
-            try:
-                if isinstance(song[field], basestring):
-                    raise TypeError
-                value = " / ".join(song[field])
-            except TypeError:
-                value = song[field]
-            if value is None: value = ''
-
+            value = song.get_tag_display(field, artist_compilations=False)
+            if value is None:
+                value = ''
             ar.append(value)
         return ar
 
@@ -389,9 +420,9 @@ class Playlist(gtk.VBox):
         """
             Returns the currently selected track
         """
-        tracks = self.get_selected_tracks()
-        if not tracks: return None
-        else: return tracks[0]
+        trs = self.get_selected_tracks()
+        if not trs: return None
+        else: return trs[0]
 
     def get_selected_tracks(self):
         """
@@ -407,6 +438,38 @@ class Playlist(gtk.VBox):
 
         return songs
 
+    def get_tracks_rating(self):
+        """
+            Returns the rating of the selected tracks in the tree view
+            Returns 0 if not all tracks have the same rating or if the selection
+            is too big
+        """
+        rating = 0
+        selection = self.list.get_selection()
+        (model, paths) = selection.get_selected_rows()
+
+        if paths != None and len(paths) > 0:
+            iter = self.model.get_iter(paths[0])
+            song = self.model.get_value(iter, 0)
+            rating = song.get_rating ()
+        else:
+            return 0 # no tracks
+
+        if rating == 0:
+            return 0 # if first song has 0 as a rating, we know the final result
+
+        if len(paths) > settings.get_option('miscellaneous/rating_widget_tracks_limit', 100):
+            return 0 # too many tracks, skipping
+
+        for path in paths:
+            iter = self.model.get_iter(path)
+            song = self.model.get_value(iter, 0)
+            if song.get_rating() != rating:
+                return 0 # different ratings
+
+        return rating # only one rating in the tracks, returning it
+
+
     def update_iter(self, iter, song):
         """
             Updates the track at "iter"
@@ -426,7 +489,8 @@ class Playlist(gtk.VBox):
         while True:
             check = self.model.get_value(iter, 0)
             if not check: break
-            if check == song or check.get_loc() == song.get_loc():
+            if check == song or \
+                    check.get_loc_for_io() == song.get_loc_for_io():
                 self.update_iter(iter, song)
                 break
             iter = self.model.iter_next(iter)
@@ -534,16 +598,20 @@ class Playlist(gtk.VBox):
 
         selection = self.list.get_selection()
         selection.set_mode(gtk.SELECTION_MULTIPLE)
+        selection.connect('changed', lambda s: self.selection_changed())
 
-        window = gtk.Window()
-        img = window.render_icon('gtk-media-play',
+        img = self.list.render_icon('gtk-media-play',
             gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.playimg = img.scale_simple(18, 18,
             gtk.gdk.INTERP_BILINEAR)
-        img = window.render_icon('gtk-media-pause',
+        img = self.list.render_icon('gtk-media-pause',
             gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.pauseimg = img.scale_simple(18, 18,
             gtk.gdk.INTERP_BILINEAR)
+        img = self.list.render_icon('gtk-stop',
+            gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.stopimg = img.scale_simple(12, 12,
+            gtk.gdk.INTERP_BILINEAR)
 
     def column_changed(self, *e):
         """
@@ -553,20 +621,21 @@ class Playlist(gtk.VBox):
         self.list.disconnect(self.changed_id)
         cols = []
         for col in self.list.get_columns():
-            cols.append(self.column_by_display[col.get_title()].id)
+            cols.append(self.column_by_display[col.get_title().
+                                               decode('utf-8')].id)
             self.list.remove_column(col)
 
         settings.set_option('gui/columns', cols)
         self._setup_columns()
         self._set_tracks(self.playlist.get_tracks())
-    
+
     def get_needs_save(self):
         return self.playlist.get_needs_save()
-    
+
     def set_needs_save(self, val=True):
         self.playlist.set_needs_save(val)
         self.emit('playlist-content-changed', self.get_needs_save())
-    
+
     def drag_data_received(self, tv, context, x, y, selection, info, etime):
         """
             Called when data is recieved
@@ -593,27 +662,30 @@ class Playlist(gtk.VBox):
         if context.action != gtk.gdk.ACTION_MOVE:
             pass
 
-        drop_info = tv.get_dest_row_at_pos(x, y)
-        if drop_info:
-            path, position = drop_info
-            iter = self.model.get_iter(path)
-            if (position == gtk.TREE_VIEW_DROP_BEFORE or
-                position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
-                first = False
-            else:
-                first = True
+        try:
+            drop_info = tv.get_dest_row_at_pos(x, y)
 
-        current_tracks = self.playlist.get_tracks()
-        (tracks, playlists) = self.list.get_drag_data(locs)
+            if drop_info:
+                path, position = drop_info
+                iter = self.model.get_iter(path)
+                if (position == gtk.TREE_VIEW_DROP_BEFORE or
+                    position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
+                    first = False
+                else:
+                    first = True
+        except AttributeError:
+            drop_info = None
+            pass
 
-        tracks = sort_tracks(tracks)
+        (trs, playlists) = self.list.get_drag_data(locs)
+
+        (column, descending) = self.get_sort_by()
+        trs = trax.sort_tracks(self.return_order_tags(column), trs, reverse=descending)
 
         # Determine what to do with the tracks
         # by default we load all tracks.
         # TODO: should we load tracks we find in the collection from there??
-        for track in tracks:
-            if not Playlist._is_drag_source and track in current_tracks:
-                continue
+        for track in trs:
             if not drop_info:
                 self._append_track(track)
             else:
@@ -640,20 +712,13 @@ class Playlist(gtk.VBox):
         else:
             context.finish(True, False, etime)
 
-        # iterates through the list and adds any tracks that are
-        # not in the playlist to the current playlist
-        current_tracks = self.playlist.get_tracks()
         iter = self.model.get_iter_first()
         if not iter:
             # Do we need to reactivate the callbacks when this happens?
             gobject.idle_add(self.add_track_callbacks)
             return
-        while True:
-            track = self.model.get_value(iter, 0)
-            if not track in current_tracks:
-                self.playlist.add_tracks((track,))
-            iter = self.model.iter_next(iter)
-            if not iter: break
+
+        self.playlist.add_tracks(trs)
 
         # Re add all of the tracks so that they
         # become ordered
@@ -669,7 +734,7 @@ class Playlist(gtk.VBox):
             iter = self.model.iter_next(iter)
             if not iter: break
 
-        if tracks:
+        if trs:
             self.set_needs_save(True)
 
         gobject.idle_add(self.add_track_callbacks)
@@ -706,34 +771,40 @@ class Playlist(gtk.VBox):
             self.model.remove(iter)
 
     def remove_selected_tracks(self):
-        self.remove_tracks(self.get_selected_tracks())
+        selection = self.list.get_selection()
+        (model, paths) = selection.get_selected_rows()
+        self.remove_rows(paths)
 
-    def remove_tracks(self, tracks):
-        event.remove_callback(self.on_remove_tracks, 'tracks_removed',
-            self.playlist)
+    def remove_rows(self, paths, playlist=True):
+        if playlist:
+            event.remove_callback(self.on_remove_tracks, 'tracks_removed',
+                    self.playlist)
+            # Make sure the callback actually gets removed before proceeding
+            event.wait_for_pending_events()
+            ranges = []
+            curstart = paths[0][0]
+            last = curstart
+            for i in paths[1:]:
+                val = i[0]
+                if val == last+1:
+                    last += 1
+                    continue
+                else:
+                    ranges.append((curstart, last))
+                    curstart = val
+                    last = val
+            ranges.append((curstart, last))
+            for start, end in ranges:
+                self.playlist.remove_tracks(start, end)
+            gobject.idle_add(event.add_callback, self.on_remove_tracks,
+                'tracks_removed', self.playlist)
 
-        # Make sure the callback actually gets removed before proceeding
-        event.wait_for_pending_events()
-        tracks = set(tracks) # for faster traversing
-
-        rows = []
-        iter = self.model.get_iter_first()
-        while iter:
-            value = self.model.get_value(iter, 0)
-            if value in tracks: rows.append(iter)
-            iter = self.model.iter_next(iter)
-
-        for row in rows:
-            track = self.model.get_value(row, 0)
-            try:
-                self.playlist.remove(self.playlist.index(track))
-            except ValueError: 
-                # track has already been removed from the playlist
-                pass
+        iters = [self.model.get_iter(x) for x in paths]
+        for row in iters:
             self.model.remove(row)
 
-        gobject.idle_add(event.add_callback, self.on_remove_tracks, 
-            'tracks_removed', self.playlist)
+        self.list.set_cursor(paths[0][0])
+
         self.emit('track-count-changed', len(self.playlist))
         self.set_needs_save(True)
 
@@ -750,12 +821,12 @@ class Playlist(gtk.VBox):
             Called when a drag source wants data for this drag operation
         """
         Playlist._is_drag_source = True
-        
-        tracks = self.get_selected_tracks()
-        for track in tracks:
-            guiutil.DragTreeView.dragged_data[track.get_loc()] = track
 
-        locs = guiutil.get_urls_for(tracks)
+        trs = self.get_selected_tracks()
+        for track in trs:
+            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
+
+        locs = guiutil.get_urls_for(trs)
         selection.set_uris(locs)
 
     def setup_model(self, map):
@@ -768,7 +839,6 @@ class Playlist(gtk.VBox):
             ar.append(str)
 
         self.model = gtk.ListStore(*ar)
-        self.model_blank = gtk.ListStore(*ar)
         self.list.set_model(self.model)
 
     def _setup_columns(self):
@@ -877,7 +947,7 @@ class Playlist(gtk.VBox):
             Called when the user resizes a column
         """
         if self._is_queue: return
-        col_struct = self.column_by_display[col.get_title()]
+        col_struct = self.column_by_display[col.get_title().decode('utf-8')]
         name = 'gui/col_width_%s' % col_struct.id
         w = col.get_width()
         if w != settings.get_option(name, -1):
@@ -891,7 +961,7 @@ class Playlist(gtk.VBox):
         """
         title = column.get_title()
         for col in self.list.get_columns():
-            if title == col.get_title():
+            if title == col.get_title().decode('utf-8'):
                 order = column.get_sort_order()
                 if order == gtk.SORT_ASCENDING:
                     order = gtk.SORT_DESCENDING
@@ -901,9 +971,10 @@ class Playlist(gtk.VBox):
                 col.set_sort_order(order)
             else:
                 col.set_sort_indicator(False)
+                col.set_sort_order(gtk.SORT_DESCENDING)
 
-        tracks = self.reorder_songs()
-        self._set_tracks(tracks)
+        trs = self.reorder_songs()
+        self._set_tracks(trs)
 
         if not self.playlist.ordered_tracks: return
         try:
@@ -911,25 +982,23 @@ class Playlist(gtk.VBox):
                 self.playlist.ordered_tracks[self.playlist.get_current_pos()]
         except IndexError:
             curtrack = self.playlist.ordered_tracks[0]
-        self.playlist.ordered_tracks = tracks
+        self.playlist.ordered_tracks = trs
         index = self.playlist.index(curtrack)
         self.playlist.set_current_pos(index)
-    
+
     def return_order_tags(self, newtag = None):
         """
             Returns a list of tags, which sorting will be used to sort tracks
             If newtag isn't set, only returns the list, otherwise adds it to the
             head of the list and saves it into the settings
         """
-        tags = settings.get_option ('gui/tags_sorting_order',
-            ['artist', 'date', 'album', 'discnumber', 'tracknumber', 'title'])
-        
+        tags = ['artist', 'date', 'album', 'discnumber', 'tracknumber', 'title']
+
         if newtag:
             if newtag in tags:
                 tags.remove (newtag)
             tags.insert (0, newtag)
-            settings.set_option ('gui/tags_sorting_order', tags)
-        
+
         return tags
 
     def reorder_songs(self):
@@ -951,19 +1020,21 @@ class Playlist(gtk.VBox):
         """
         for col in self.list.get_columns():
             if col.get_sort_indicator():
-                return (self.column_by_display[col.get_title()].id,
+                return (self.column_by_display[col.get_title().
+                                               decode('utf-8')].id,
                     col.get_sort_order() == gtk.SORT_DESCENDING)
-        return 'tracknumber', False
+        return None, False
 
     def icon_data_func(self, col, cell, model, iter):
         """
             Sets track status (playing/paused/queued) icon
         """
-
+        path = model.get_path(iter)
         item = model.get_value(iter, 0)
         image = None
 
-        if item == self.player.current:
+        if path[0] == self.playlist.get_current_pos():
+#        if item == self.player.current:
             if self.player.is_playing():
                 image = self.playimg
             elif self.player.is_paused():
@@ -977,29 +1048,24 @@ class Playlist(gtk.VBox):
 
         cell.set_property('pixbuf', image)
 
-
     def stop_icon_data_func(self, col, cell, model, iter):
         """
             Sets "stop after this" icon
         """
-
         item = model.get_value(iter, 0)
         image = None
-
-        window = gtk.Window()
         if item == self.queue.stop_track:
-            image = window.render_icon('gtk-stop',
-                gtk.ICON_SIZE_MENU)
-            image = image.scale_simple(12, 12, gtk.gdk.INTERP_BILINEAR)
-
+            image = self.stopimg
         cell.set_property('pixbuf', image)
 
-
     def set_cell_weight(self, cell, item):
         """
             Sets a CellRendererText's "weight" property according to whether
             `item` is the currently playing track.
         """
+        # Doesn't play well with multiple track instances
+        # as the passed-in information doesn't let us get the index,
+        # which we need to discriminate among instances.
         if item == self.player.current:
             weight = pango.WEIGHT_HEAVY
         else:
@@ -1009,7 +1075,7 @@ class Playlist(gtk.VBox):
     def press_header(self, widget, event):
         if event.button != 3:
             return False
-        menu = self.xml.get_widget('columns_menu_menu')
+        menu = self.builder.get_object('columns_menu_menu')
         menu.popup(None, None, None, event.button, event.time)
         return True
 
@@ -1018,9 +1084,9 @@ class Playlist(gtk.VBox):
             Debug - prints the current playlist to stdout
         """
         print banner
-        tracks = self.playlist.get_tracks()
-        for track in tracks:
-            print track.get_loc()
+        trs = self.playlist.get_tracks()
+        for track in trs:
+            print track.get_loc_for_display()
         print '---Done printing playlist'
 
     def update_rating(self, w, e):
@@ -1033,7 +1099,6 @@ class Playlist(gtk.VBox):
         left_edge = 0
         steps = settings.get_option("miscellaneous/rating_steps", 5)
         icon_size = rating._rating_width / steps
-        cols = self.list.get_columns()
         i = 0
         #calculate rating column size and position
         for col in self.append_map:
@@ -1053,30 +1118,14 @@ class Playlist(gtk.VBox):
                 leftpadding = (rating_col_width - rating._rating_width) / 2
                 i = int(math.ceil((x-left_edge-leftpadding)/icon_size))
                 new_rating = float((100*i)/steps)
-                if track['__rating'] == new_rating:
-                    track['__rating'] = 0.0
+                if track.get_tag_raw('__rating') == new_rating:
+                    track.set_tag_raw('__rating', 0.0)
                 else:
-                    track['__rating'] = new_rating
+                    track.set_tag_raw('__rating', new_rating)
                 if hasattr(w, 'queue_draw'):
                     w.queue_draw()
                 event.log_event('rating_changed', self, i)
 
-def sort_tracks(tracks):
-    from xlgui import main
-
-    pl = main.get_selected_playlist()
-    column, descending = pl.get_sort_by()
-    
-    if column != 'tracknumber':
-        tracks.sort(key=lambda track: track.sort_param(column),
-            reverse=descending)
-    else:
-        tracks = trackdb.sort_tracks(
-            ('artist', 'date', 'album', 'discnumber', 'tracknumber'),
-            tracks)
-        if descending: tracks.reverse()
-
-    return tracks
 
 class ConfirmCloseDialog(gtk.MessageDialog):
     """
@@ -1100,3 +1149,5 @@ class ConfirmCloseDialog(gtk.MessageDialog):
         response = gtk.Dialog.run(self)
         self.hide()
         return response
+
+# vim: et sts=4 sw=4
