@@ -1,6 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
-#
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2009 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,26 +15,21 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
-# from your version.
-#
-#
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import gtk.gdk, hashlib, os, pango
+import hashlib
+import os
+
+import gtk.gdk
+import pango
+
 from xl.nls import gettext as _
 
 class PrefsItem(object):
@@ -47,7 +40,7 @@ class PrefsItem(object):
     def __init__(self, prefs, widget):
         """
             Initializes the preferences item
-            expects the name of the widget in the .glade file, the default for
+            expects the name of the widget in the designer file, the default for
             this setting, an optional function to be called when the value is
             changed, and an optional function to be called when this setting
             is applied
@@ -57,21 +50,27 @@ class PrefsItem(object):
         self.prefs = prefs
 
         self._set_value()
-        if hasattr(self, 'change'):
-            self._setup_change()
+        self._setup_change()
+
+    def change(self, *args):
+        self.apply()
 
     def _setup_change(self):
         """
             Sets up the function to be called when this preference is changed
         """
         self.widget.connect('focus-out-event',
-            self.change, self.name, unicode(self.widget.get_text(), 'utf-8'))
-        self.widget.connect('activate',
-            lambda *e: self.change(self.widget, None, self.name,
-                unicode(self.widget.get_text(), 'utf-8')))
+            self.change, self.name, self._get_value())
+
+        try:
+            self.widget.connect('activate',
+                lambda *e: self.change(self.widget, None, self.name,
+                    self._get_value()))
+        except TypeError:
+            pass
 
     def _set_value(self):
-        """ 
+        """
             Sets the GUI widget up for this preference
         """
         if not self.widget:
@@ -109,6 +108,7 @@ class HashedPrefsItem(PrefsItem):
 
     def change(self, *e):
         self._dirty = True
+        self.apply()
 
     def apply(self, value=None):
         """
@@ -169,14 +169,14 @@ class DirPrefsItem(PrefsItem):
             self.prefs.settings.get_option(self.name, self.default))
         if not os.path.exists(directory):
             os.makedirs(directory)
-        self.widget.set_filename(directory)
+        self.widget.set_current_folder(directory)
 
     def _get_value(self):
         return self.widget.get_filename()
 
 
 class OrderListPrefsItem(PrefsItem):
-    """ 
+    """
         A list box with reorderable items
     """
     def __init__(self, prefs, widget):
@@ -295,9 +295,19 @@ class SelectionListPrefsItem(PrefsItem):
         move_panel = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0.0)
         move_panel.add(move_box)
 
-        widget.pack_start(available_tree)
+        available_scrollwindow = gtk.ScrolledWindow()
+        available_scrollwindow.set_property('shadow-type', gtk.SHADOW_IN)
+        available_scrollwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        available_scrollwindow.add(available_tree)
+
+        selected_scrollwindow = gtk.ScrolledWindow()
+        selected_scrollwindow.set_property('shadow-type', gtk.SHADOW_IN)
+        selected_scrollwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        selected_scrollwindow.add(selected_tree)
+
+        widget.pack_start(available_scrollwindow)
         widget.pack_start(control_panel, expand=False)
-        widget.pack_start(selected_tree)
+        widget.pack_start(selected_scrollwindow)
         widget.pack_start(move_panel, expand=False)
 
         add_button.connect('clicked', self.on_add_button_clicked)
@@ -428,7 +438,7 @@ class SelectionListPrefsItem(PrefsItem):
         if not event.state & gtk.gdk.MOD1_MASK: return
 
         keyname = gtk.gdk.keyval_name(event.keyval)
-        
+
         if keyname == 'Left':
             self.on_remove_button_clicked(None)
         elif keyname == 'Up':
@@ -617,7 +627,7 @@ class ShortcutListPrefsItem(PrefsItem):
             if accel:
                 option[action] = accel
             iter = self.list.iter_next(iter)
-        
+
         return option
 
     def update_list(self, items):
@@ -663,7 +673,7 @@ class TextViewPrefsItem(PrefsItem):
             self.prefs.settings.get_option(self.name,
             default=self.default)))
 
-    def _get_value(self):    
+    def _get_value(self):
         """
             Applies the setting
         """
@@ -678,7 +688,7 @@ class ListPrefsItem(PrefsItem):
         PrefsItem.__init__(self, prefs, widget)
 
     def _set_value(self):
-        items = self.prefs.settings.get_option(self.name, 
+        items = self.prefs.settings.get_option(self.name,
             default=self.default)
         try:
             items = " ".join(items)
@@ -689,6 +699,7 @@ class ListPrefsItem(PrefsItem):
     def _get_value(self):
         # shlex is broken with unicode, so we feed it UTF-8 and decode
         # afterwards.
+        import shlex
         values = shlex.split(self.widget.get_text())
         values = [unicode(value, 'utf-8') for value in values]
         return values
@@ -702,7 +713,7 @@ class SpinPrefsItem(PrefsItem):
         PrefsItem.__init__(self, prefs, widget)
 
     def _set_value(self):
-        value = self.prefs.settings.get_option(self.name, 
+        value = self.prefs.settings.get_option(self.name,
             default=self.default)
         self.widget.set_value(value)
 
@@ -728,7 +739,7 @@ class FloatPrefsItem(PrefsItem):
 
     def _set_value(self):
         self.widget.set_text(str(
-            self.prefs.settings.get_option(self.name, 
+            self.prefs.settings.get_option(self.name,
             default=self.default)))
 
     def _get_value(self):
@@ -753,12 +764,12 @@ class ColorButtonPrefsItem(PrefsItem):
 
     def _set_value(self):
         self.widget.set_color(gtk.gdk.color_parse(
-            self.prefs.settings.get_option(self.name, 
+            self.prefs.settings.get_option(self.name,
             self.default)))
 
     def _get_value(self):
         color = self.widget.get_color()
-        string = "#%.2x%.2x%.2x" % (color.red / 257, color.green / 257, 
+        string = "#%.2x%.2x%.2x" % (color.red / 257, color.green / 257,
             color.blue / 257)
         return string
 
@@ -774,10 +785,10 @@ class FontButtonPrefsItem(ColorButtonPrefsItem):
         self.widget.connect('font-set', self.change)
 
     def _set_value(self):
-        font = self.prefs.settings.get_option(self.name, 
+        font = self.prefs.settings.get_option(self.name,
             self.default)
         self.widget.set_font_name(font)
-        
+
     def _get_value(self):
         font = self.widget.get_font_name()
         return font
@@ -796,7 +807,7 @@ class ComboPrefsItem(PrefsItem):
         self.widget.connect('changed', self.change)
 
     def _set_value(self):
-        item = self.prefs.settings.get_option(self.name, 
+        item = self.prefs.settings.get_option(self.name,
             self.default)
 
         if self.use_map:
@@ -806,7 +817,7 @@ class ComboPrefsItem(PrefsItem):
             return
 
         if self.use_index:
-            index = self.prefs.settings.get_option(self.name, 
+            index = self.prefs.settings.get_option(self.name,
                 self.default)
             self.widget.set_active(index)
             return
